from .client import ClientOpenAPIResponseFactory
from .starlette import StarletteOpenAPIResponseFactory

ClientOpenAPIResponse = ClientOpenAPIResponseFactory.create
StarletteOpenAPIResponse = StarletteOpenAPIResponseFactory.create
